##############################################################################
#
# Generic Kultur configuration
#
##############################################################################
#
# Copyright 2009 Tim Brody <tdb2@ecs.soton.ac.uk>, University of Southampton
#
# This file is released under the GPLv3 license.
#

my $kultur = {};
$c->{kultur} = $kultur;

# thumbnail geometries to make available
my $geoms = {
	small => [66,50],
	medium => [200,150],
	preview => [400,300],
	lightbox => [640,480],
};
$c->{kultur}->{geoms} = $geoms;

# maximum width (in pixels) for each tab
$c->{kultur}->{max_width} = 700;

# maxiumum height (in pixels) for each tab (may overflow)
$c->{kultur}->{max_height} = 600;

##############################################################################
# End of Configuration
##############################################################################


# generate thumbnails in order from biggest to smallest
# (possible future optimisation)
$c->{kultur}->{geom_keys} = [sort {
		$geoms->{$b}->[0] <=> $geoms->{$a}->[1]
	} keys %$geoms];

# tell eprints to generate all of our thumbnail geometries
$c->{thumbnail_types} = sub {
	my( $list, $session, $doc ) = @_;

	@$list = @{$session->get_repository->get_conf( "kultur", "geom_keys" )};
};

# ThumbnailImages plugin settings
$c->{plugins}->{"Export::KulturPreviews"}->{params}->{disable} = 0;
$c->{plugins}->{"Convert::Kultur"}->{params}->{disable} = 0;
$c->{plugins}->{"Convert::Kultur"}->{params}->{sizes} = $geoms;
$c->{plugins}->{"Convert::ImageMagick::ThumbnailImages"}->{params}->{disable} = 1;
$c->{plugins}->{"InputForm::Component::KulturRenderDocuments"}->{params}->{disable} = 0;

# check these fields aren't already defined
for(@{$c->{fields}->{document}})
{
	if( $_->{name} eq "thumbnail_picker" )
	{
		EPrints::abort( "Can't add required kultur document field 'thumbnail_picker' - already defined" );
	}
	elsif( $_->{name} eq "media" )
	{
		EPrints::abort( "Can't add required kultur document field 'media' - already defined" );
	}
}

push @{$c->{fields}->{document}},
# this metadata field can be included in the eprint workflow to provide a
# thumbnail picker widget
	{
		name => "thumbnail_picker",
		type => "thumbpicker",
	},

# this metadata field adds technical data about the media stream
	{
		name => "media",
		type => "compound",
		fields => [
# offset from start in seconds.frames to grab a sample
			{ sub_name => "sample_start", type => "text" },
# reserved
			{ sub_name => "sample_end", type => "text" },
# duration of the media, in seconds.frames
			{ sub_name => "duration", type => "text" },
# audio codec
			{ sub_name => "audio_codec", type => "text" },
# video codec
			{ sub_name => "video_codec", type => "text" },
# vertical lines of video (e.g. PAL=480 lines)
			{ sub_name => "height", type => "int", },
# horizontal lines of video (e.g. PAL=640 lines)
			{ sub_name => "width", type => "int", },
# aspect ratio (pixels may be non-square) - formatted as 3:4
			{ sub_name => "aspect_ratio", type => "text", },
		],
	};

=item kultur_set_document_automatic_fields( $doc )

Set the media field based on the main file in $doc. If the user has picked a different thumbnail regenerate thumbnails.

=cut

sub kultur_set_document_automatic_fields
{
	my( $doc ) = @_;

# media field settings
	if( $doc->get_dataset->has_field( "media" ) && $doc->is_set( "main" ) )
	{
		my $value = EPrints::Utils::clone( $doc->get_value( "media" ) );
# thumbnail picker induced change
		if( exists($doc->{changed}->{"thumbnail_picker"}) )
		{
			$value->{"sample_start"} = $doc->get_value( "thumbnail_picker" );
			$doc->set_value( "media", $value );
		}

# main file changed, re-populate the media field
		if( exists($doc->{changed}->{"main"}) )
		{
			my $plugin = $doc->get_session->plugin( "Convert::Kultur" );
			if( defined $plugin )
			{
				my $src = $doc->local_path."/".$doc->get_main;
				my $settings = $plugin->get_dimensions( $src );
				for(qw( duration audio_codec video_codec width height aspect_ratio ))
				{
					$value->{$_} = $settings->{$_};
				}
				if( !defined $value->{"duration"} || $value->{"duration"} == 0 )
				{
					$value->{"sample_start"} = undef;
				}
				elsif 
				  (
					defined $value->{"sample_start"}
					&& int($value->{"sample_start"}) > $value->{"duration"}
				  )
				{
					$value->{"sample_start"} = 0;
				}
				$doc->set_value( "media", $value );
			}
		}

# if the sample time changed recreate the thumbnails
		if(
			exists($doc->{changed}->{"media_sample_start"})
			&& EPrints::Utils::is_set( $value->{"sample_start"} ) 
		  )
		{
			$doc->queue_files_modified();
		}
	}
	elsif( !$doc->get_dataset->has_field( "media" ) )
	{
		$doc->{session}->get_repository( "Log: Kultur document media field extension installed without document media being available" );
	} # end of media field
}

=item $docs = kultur_random_documents( $session, $max, $size )

Returns psuedo-random documents from the repository that have a thumbnail available in $size. Returns up to $max documents.

=cut

sub kultur_random_documents
{
	my( $session, $max, $size ) = @_;

	my $docs = [];

	my $db = $session->get_database();

	my $ids = _mysql_rand_docs( $db, $max*100 );

	my %seen;
	for(@$ids)
	{
		my( $eprintid, $docid ) = @$_;
		next if $seen{$eprintid};
		$seen{$eprintid} = 1;

		my $doc = EPrints::DataObj::Document->new( $session, $docid );
		next unless defined $doc;

		next unless $doc->thumbnail_url( $size );

		push @$docs, $doc;
		last if --$max == 0;
	}

	return $docs;
}

sub _mysql_rand_docs
{
	my( $db, $max ) = @_;

	my @ids;
	
	#3.2
	my $sql = "SELECT eprintid,document.docid,relation_type FROM eprint INNER JOIN document USING(eprintid) LEFT JOIN document_relation_type ON document.docid=document_relation_type.docid AND document_relation_type.relation_type='http://eprints.org/relation/isVolatileVersionOf' WHERE security='public' AND eprint_status='archive' AND document_relation_type.docid IS NULL ORDER BY RAND() LIMIT $max";
	#3.1
	#my $sql = "SELECT eprintid,docid FROM document INNER JOIN eprint USING(eprintid) WHERE security='public' AND eprint_status='archive' ORDER BY RAND() LIMIT $max";
	my $sth = $db->prepare( $sql );
	$sth->execute;

	while(my @row = $sth->fetchrow_array)
	{
		push @ids, \@row;
	}

	return \@ids;
}

sub EPrints::Script::Compiled::run_kultur_slideshow
{
	my( $self, $state, $max ) = @_;

	if( $max->[1] ne "INTEGER" )
	{
		EPrints::abort( "Usage: kultur_slideshow( MAX )" );
	}

	$max = $max->[0];

	my $size = 'preview';

	my $session = $state->{session};

	my $docs = kultur_random_documents( $session, $max, $size );

	my $frag = $session->make_doc_fragment;

	my $div = $session->make_element( "div", id => "slideshow" );
	$frag->appendChild( $div );
	my $div_cnt = $session->make_element( "div", id => "slideshow_cnt" );
	$div->appendChild( $div_cnt );
	my $div_idx = $session->make_element( "div", id => "slideshow_idx", style => "display: none" );
	$div->appendChild( $div_idx );

	my $idx_ul = $session->make_element( "ul" );
	$div_idx->appendChild( $idx_ul );

	my $first = 1;
	foreach my $doc (@$docs)
	{
		my $eprint = $doc->get_eprint;

		# index icon
		my $li = $session->make_element( "li",
			class => "icon",
			title => $eprint->value( "title" ),
		);
		$idx_ul->appendChild( $li );
		$li->appendChild( $session->make_element( "img",
			src => $doc->icon_url( size => "small" )
			) );

		# slide
		my $div = $session->make_element( "div", class => "slide" );
		$div_cnt->appendChild( $div );
		if( $first )
		{
			$first = 0;
			$div->setAttribute( style => "display: block" );
		}
		else
		{
			$div->setAttribute( style => "display: none" );
			$li->setAttribute( style => "opacity: .5; filter: alpha(opacity=50); -moz-opacity: .5; -khtml-opacity: .5;" );
		}

		my $abs_link = $session->render_link( $doc->get_eprint->get_url );
		$div->appendChild( $abs_link );
		my $img = $session->make_element( "img",
			src => $doc->thumbnail_url( $size )
		);
		$abs_link->appendChild( $img );
		$div->appendChild( $session->make_element( "br" ) );

		my $caption = $eprint->render_citation_link( "default" );
		$div->appendChild( $caption );
	}

	return [ $frag, "XHTML" ];
}
