package EPrints::Plugin::Export::KulturPreviews;

use EPrints::Plugin::Export;
@ISA = ( "EPrints::Plugin::Export" );

use strict;

sub new
{
	my( $class, %opts ) = @_;

	my $self = $class->SUPER::new( %opts );

	$self->{name} = "Video and Audio Previews";
	$self->{accept} = [ 'list/document' ];
	$self->{visible} = "";
	$self->{suffix} = ".txt";
	$self->{mimetype} = "text/plain; charset=utf-8";
	
	return $self;
}

sub output_list
{
	my( $plugin, %opts ) = @_;

	my $convertor = $plugin->{session}->plugin( "Convert::Kultur" );

	my $output = "";

	$opts{list}->map(sub {
		my( $session, undef, $doc ) = @_;

		return unless $doc->exists_and_set( "media" );
		my $media = $doc->get_value( "media" );
		return unless defined $media->{duration};
		return unless $media->{duration} > 0;

		my $size = "video_preview";
		if( $convertor->is_video( $doc ) )
		{
			$size = "video_preview";
		}
		else
		{
			$size = "audio_preview";
		}
		return if $doc->has_related_objects( EPrints::Utils::make_relation( "has${size}Version" ) );
		my $main = $doc->get_main;
		if( !$convertor->can_convert( $doc ) )
		{
			$output .= "Convertor can't convert document ".$doc->get_id."\n";
			if( $opts{fh} )
			{
				print {$opts{fh}} "Convertor can't convert document ".$doc->get_id."\n";
			}
			return;
		}
		my $thumbnail = $convertor->convert( $doc->parent, $doc, "thumbnail_$size" );
		if( defined $thumbnail )
		{
			$doc->add_object_relations( $thumbnail,
				EPrints::Utils::make_relation( "has${size}Version" ) => EPrints::Utils::make_relation( "is${size}VersionOf" ) );
			$doc->commit;
			$thumbnail->commit;
			$output .= "Generated: document.".$thumbnail->id."\n";
			if( $opts{fh} )
			{
				print {$opts{fh}} "Generated: document.".$thumbnail->id."\n";
			}
		}
		else
		{
			$output .= "Failed to convert document ".$doc->get_id.": $main\n";
			if( $opts{fh} )
			{
				print {$opts{fh}} "Failed to convert document ".$doc->get_id.": $main\n";
			}
		}
	});

	return $output;
}

1;
