package EPrints::Plugin::InputForm::Component::KulturRenderDocuments;

=head1 NAME

EPrints::Plugin::InputForm::Component::KulturRenderDocuments - Thumbnail previews in the workflow

=head1 DESCRIPTION

This module renders thumbnails of all documents in an eprint as an "InputForm"
component. This allows you to add document thumbnails to any part of an eprint
workflow.

=head1 USAGE

Copy this module to your plugins directory and add this to your Workflow:

  	<component type="KulturRenderDocuments"></component>

E.g.

  <stage name="details">
  	<component type="KulturRenderDocuments"></component>
    <component><field ref="contact_email"/></component>
	...

=cut

use EPrints::Plugin::InputForm::Component;

@ISA = ( "EPrints::Plugin::InputForm::Component" );

use strict;

sub new
{
	my( $class, %opts ) = @_;

	my $self = $class->SUPER::new( %opts );

	$self->{name} = "KulturRenderDocuments";
	$self->{visible} = "all";

	return $self;
}

sub render_content
{
	my( $self ) = @_;

	my $session = $self->{session};

	my $eprint = $self->{dataobj};

	my $div = $session->make_element( "div", class=>"ep_workflow_documents" );

	foreach my $doc ($eprint->get_all_documents)
	{
		my $small_url = $doc->thumbnail_url( "small" );
		next unless defined $small_url;
		my $link = $session->render_link( $doc->thumbnail_url( "lightbox" ) );
		$link->setAttribute( "rel", "lightbox[documents]" );
		my $title = $doc->render_citation_link( "lightbox" );
		$link->setAttribute( "title", EPrints::XML::to_string( $title ) );
		EPrints::XML::dispose( $title );
		$div->appendChild( $link );
		my $img = $session->make_element( "img",
			src => $small_url,
			border => "0",
		);
		$link->appendChild( $img );
		$div->appendChild( $session->make_text( " " ) );
	}

	return $div;
}

sub render_help
{
	my( $self, $surround ) = @_;
	
	return $self->html_phrase( "help", placeholding => $self->{session}->make_text( $self->{placeholding} ) );
}

sub render_title
{
	my( $self, $surround ) = @_;

	return $self->html_phrase( "title", placeholding => $self->{session}->make_text( $self->{placeholding} ) );
}
	
1;





