
/* 
 * This javascript file will be loaded after all the system files.
 * (so other javascript functions will already be loaded)
 * 
 * Javascript files are loaded in alphabetic order, hence the "90" 
 * in the filename to force it to load after the other files!
 * 
 * To totally replace a system js file, create a file of the same 
 * name in this directory. eg. 50_preview.js
 * 
*/

SlideShowOptions = Object.extend({
	height: 450,
	width: 640,
	transitionDuration: 1.5,
	iconHeight: 50
}, window.SlideShowOptions || {});

var SlideShow = Class.create();

SlideShow.prototype = {
	slideArray: [],
	slideIconArray: [],
	activeSlide: -1, // nextSlide == 0
	autoChanger: undefined,

	initialize: function() {
		var slideshow = $('slideshow');
		var slideshow_idx = $('slideshow_idx');

		if( slideshow == null || slideshow_idx == null )
			return;

		slideshow_idx.style.display = 'block';
		slideshow_idx.show();

		this.slideArray = $$('#slideshow .slide');
		this.slideIconArray = $$('#slideshow .icon');

		if( this.slideArray.length == 0 || this.slideIconArray == 0 )
			return;

		for(var i = 0; i < this.slideIconArray.length; ++i)
		{
			var cur = this.slideIconArray[i];

			this.addIcon( slideshow_idx, cur, i );
		}

		this.activeSlide = 0;
		this.start(10);
		this.slideIconArray[0].onload = this.nextSlide.bind(this);
	},

	addIcon: function(cnt, icon, idx) {
		/* hmm, expected an image */
		var img = icon.select( 'img' ).first();
		if( img == null || img.nodeName != 'IMG' )
		{
			alert("Expected IMG in icon in slideshow");
			return;
		}

		function handler(e) {
			var data = $A(arguments);
			data.shift();
			this.stop();
			this.changeSlide(data[0]);
		}
		Event.observe(icon, "click", handler.bindAsEventListener(this, idx));

		/* the first slide is always shown */
/*		if( idx != 0 )
		{
			img.onload = (function(){
				this.hideIcon( idx );
			}).bind(this);
		} */
	},

	hideIcon: function( i ) {
		var icon = this.slideIconArray[i];
		new Effect.Opacity( icon, { from: 1.0, to: 0.5, duration: 0.5 });
	},

	showIcon: function( i ) {
		var icon = this.slideIconArray[i];
		new Effect.Opacity( icon, { from: 0.5, to: 1.0, duration: 0.5 });
	},

	hideSlide: function( i ) {
		var slide = this.slideArray[i];
		slide.fade({ duration: SlideShowOptions.transitionDuration });
	},

	showSlide: function( i ) {
		var slide = this.slideArray[i];
		slide.appear({ duration: SlideShowOptions.transitionDuration });
	},

	start: function(t) {
		this.stop();
		this.autoChanger = new PeriodicalExecuter(
				this.nextSlide.bind(this),
				t
			);
	},

	stop: function() {
		if( !this.autoChanger )
			return;
		this.autoChanger.stop();
		this.autoChanger = undefined;
	},

	changeSlide: function(i) {
		if( i == this.activeSlide || i < 0 || i >= this.slideArray.length )
			return;
		if( this.activeSlide > -1 )
		{
			this.hideIcon( this.activeSlide );
			this.hideSlide( this.activeSlide );
		}

		this.showIcon( i );
		this.showSlide( i );

		this.activeSlide = i;
	},

	nextSlide: function() {
		this.changeSlide((this.activeSlide+1) % this.slideArray.length);
	}
}

document.observe('dom:loaded', function () { new SlideShow(); });

