package EPrints::MetaField::Thumbpicker;

use EPrints::MetaField::Text;
@ISA = qw( EPrints::MetaField::Text );

use strict;

sub form_value_single
{
	my( $self, $session, $basename, $object ) = @_;

	my $basename_secs = "${basename}_seconds";
	my $secs = $session->param( $basename_secs );
	# user setting overrides picker
	if( EPrints::Utils::is_set( $secs ) )
	{
		return $secs;
	}

	return $session->param( $basename );
}

sub get_input_elements
{
	my( $self, $session, $value, $staff, $obj, $basename ) = @_;	

	Carp::confess( "Object not defined" ) unless defined $obj;

	my $fn = $obj->get_main();
	return [ [ ] ] if !defined $fn;

	return [ [ ] ] if !$obj->exists_and_set( "media_duration" );
	my $duration = $obj->get_value( "media_duration" );
	return [ [ ] ] if $duration == 0;

	return [ [ ] ] if !$obj->is_set( "media_video_codec" );

	my $plugin = $session->plugin( "Convert::Kultur" );
	return [ [ ] ] if !defined $plugin;

	my $seconds_id = "${basename}_seconds";

	my $table = $session->make_element( "table" );
	my $tr_top = $table->appendChild( $session->make_element( "tr" ) );
	my $tr_bottom = $table->appendChild( $session->make_element( "tr" ) );
	my $td;

	my $user_js = "";

	# picker
	my $previous_seconds = -1;
	foreach my $offset ("0","10","33%","66%","100%")
	{
		my $seconds = int($plugin->calculate_offset( $duration, $offset ));
		next if $seconds <= $previous_seconds;
		$previous_seconds = $seconds;

		my $id = "${basename}_radio_${seconds}";

		my $picker_js = "\$('$seconds_id').value = '$seconds'";
		$user_js .= "\$('$id').checked = false;";

		my $url = $session->get_url( path => "cgi", "users/video_cell" );
		$url = URI::http->new( $url );
		$url->query_form(
			docid => $obj->get_id,
			sample_start => $seconds,
		);

		my $input = $session->make_element( "input", 
			type => "radio",
			name => $basename,
			value => $seconds,
			id => $id,
			onclick => $picker_js );
		if( defined $value && $seconds eq $value )
		{
			$input->setAttribute( checked => "yes" );
		}
		$td = $session->make_element( "td", align => "center", valign => "middle" );
		$tr_bottom->appendChild( $td );
		$td->appendChild( $input );
		$td = $session->make_element( "td", align => "center", valign => "bottom" );
		$tr_top->appendChild( $td );
		my $label = $session->make_element( "label", for => $id );
		$td->appendChild( $label );
		$label->appendChild( $session->make_element( "img",
			src => $url
			) );
	}

	# user-supplied seconds offset
	$td = $session->make_element( "td", align => "center", valign => "bottom" );
	$tr_top->appendChild( $td );
	my $label = $session->make_element( "label", for => $seconds_id );
	$td->appendChild( $label );
	$label->appendChild( $session->html_phrase( $self->get_name."_seconds" ) );
	$td = $session->make_element( "td", align => "center" );
	$tr_bottom->appendChild( $td );
	my $input = $session->make_element( "input",
		name => $seconds_id,
		id => $seconds_id,
		type => "text",
		size => "5",
		value => $value,
		onkeypress => $user_js,
		);
	$td->appendChild( $input );

	return [ [ {el=>$table} ] ];
}


1;
